import unittest
import amulet_nbt


class LegacyNBTTests(unittest.TestCase):
    def test_legacy(self):
        self.assertIs(amulet_nbt.ByteTag, amulet_nbt.TAG_Byte)
        self.assertIs(amulet_nbt.ShortTag, amulet_nbt.TAG_Short)
        self.assertIs(amulet_nbt.IntTag, amulet_nbt.TAG_Int)
        self.assertIs(amulet_nbt.LongTag, amulet_nbt.TAG_Long)
        self.assertIs(amulet_nbt.FloatTag, amulet_nbt.TAG_Float)
        self.assertIs(amulet_nbt.DoubleTag, amulet_nbt.TAG_Double)
        self.assertIs(amulet_nbt.StringTag, amulet_nbt.TAG_String)
        self.assertIs(amulet_nbt.ListTag, amulet_nbt.TAG_List)
        self.assertIs(amulet_nbt.CompoundTag, amulet_nbt.TAG_Compound)
        self.assertIs(amulet_nbt.ByteArrayTag, amulet_nbt.TAG_Byte_Array)
        self.assertIs(amulet_nbt.IntArrayTag, amulet_nbt.TAG_Int_Array)
        self.assertIs(amulet_nbt.LongArrayTag, amulet_nbt.TAG_Long_Array)


if __name__ == "__main__":
    unittest.main()



# Base Types
import warnings
import re

from . import _version

__version__ = _version.get_versions()["version"]
__major__ = int(re.match(r"\d+", __version__)[0])
__all__ = [
    "AbstractBaseTag",
    "AbstractBaseImmutableTag",
    "AbstractBaseMutableTag",
    "AbstractBaseNumericTag",
    "AbstractBaseIntTag",
    "ByteTag",
    "ShortTag",
    "IntTag",
    "LongTag",
    "ByteTag",
    "TAG_Byte",
    "ShortTag",
    "TAG_Short",
    "IntTag",
    "TAG_Int",
    "LongTag",
    "TAG_Long",
    "AbstractBaseFloatTag",
    "FloatTag",
    "DoubleTag",
    "FloatTag",
    "TAG_Float",
    "DoubleTag",
    "TAG_Double",
    "AbstractBaseArrayTag",
    "ByteArrayTag",
    "IntArrayTag",
    "LongArrayTag",
    "ByteArrayTag",
    "TAG_Byte_Array",
    "IntArrayTag",
    "TAG_Int_Array",
    "LongArrayTag",
    "TAG_Long_Array",
    "StringTag",
    "StringTag",
    "TAG_String",
    "NamedTag",
    "ListTag",
    "ListTag",
    "TAG_List",
    "CompoundTag",
    "CompoundTag",
    "TAG_Compound",
    "load",
    "load_many",
    "ReadContext",
    "from_snbt",
    "NBTError",
    "NBTLoadError",
    "NBTFormatError",
    "SNBTParseError",
    "SNBTType",
    "IntType",
    "FloatType",
    "NumberType",
    "ArrayType",
    "AnyNBT",
    "utf8_decoder",
    "utf8_encoder",
    "utf8_escape_decoder",
    "utf8_escape_encoder",
]

from ._value import (
    AbstractBaseTag,
    AbstractBaseImmutableTag,
    AbstractBaseMutableTag,
)
from ._numeric import AbstractBaseNumericTag

# Types
from ._int import (
    AbstractBaseIntTag,
    ByteTag,
    ShortTag,
    IntTag,
    LongTag,
    ByteTag as TAG_Byte,
    ShortTag as TAG_Short,
    IntTag as TAG_Int,
    LongTag as TAG_Long,
)
from ._float import (
    AbstractBaseFloatTag,
    FloatTag,
    DoubleTag,
    FloatTag as TAG_Float,
    DoubleTag as TAG_Double,
)
from ._array import (
    AbstractBaseArrayTag,
    ByteArrayTag,
    IntArrayTag,
    LongArrayTag,
    ByteArrayTag as TAG_Byte_Array,
    IntArrayTag as TAG_Int_Array,
    LongArrayTag as TAG_Long_Array,
)
from ._string import (
    StringTag,
    StringTag as TAG_String,
)

from ._named_tag import NamedTag

from ._list import (
    ListTag,
    ListTag as TAG_List,
)
from ._compound import (
    CompoundTag,
    CompoundTag as TAG_Compound,
)

# Load functions
from ._load_nbt import load, load_many, ReadContext
from ._load_snbt import from_snbt

from ._errors import NBTError, NBTLoadError, NBTFormatError, SNBTParseError

from ._dtype import SNBTType, IntType, FloatType, NumberType, ArrayType, AnyNBT

from ._util import utf8_decoder, utf8_encoder, utf8_escape_decoder, utf8_escape_encoder


def __getattr__(name):
    if name == "NBTFile":
        warnings.warn(
            "NBTFile is depreciated. Use NamedTag instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return NamedTag
    elif name == "BaseArrayType":
        warnings.warn(
            "BaseArrayType is depreciated. Use AbstractBaseArrayTag instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return AbstractBaseArrayTag
    elif name == "BaseValueType":
        warnings.warn(
            "BaseValueType is depreciated. Use AbstractBaseTag instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        return AbstractBaseTag
    raise AttributeError(f"module {__name__} has no attribute {name}")



import unittest
import numpy
import os
from amulet_nbt import (
    ByteTag,
    ShortTag,
    IntTag,
    LongTag,
    FloatTag,
    DoubleTag,
    ByteArrayTag,
    IntArrayTag,
    LongArrayTag,
    StringTag,
    ListTag,
    CompoundTag,
    NamedTag,
    load,
)


class MassiveNBTTests(unittest.TestCase):
    def test_api(self):
        compound = CompoundTag()
        named_compound = NamedTag(compound, name="hello")

        # the nbt objects with no inputs
        compound["emptyByte"] = ByteTag()
        compound["emptyShort"] = ShortTag()
        compound["emptyInt"] = IntTag()
        compound["emptyLong"] = LongTag()
        compound["emptyFloat"] = FloatTag()
        compound["emptyDouble"] = DoubleTag()
        compound["emptyByteArray"] = ByteArrayTag()
        compound["emptyString"] = StringTag()
        compound["emptyList"] = ListTag()
        compound["emptyCompound"] = CompoundTag()
        compound["emptyIntArray"] = IntArrayTag()
        compound["emptyLongArray"] = LongArrayTag()

        # the nbt objects with zero or empty inputs (pure python only)
        compound["zeroByte"] = ByteTag(0)
        compound["zeroShort"] = ShortTag(0)
        compound["zeroInt"] = IntTag(0)
        compound["zeroLong"] = LongTag(0)
        compound["zeroFloat"] = FloatTag(0)
        compound["zeroDouble"] = DoubleTag(0)
        compound["zeroByteArray"] = ByteArrayTag([])
        compound["zeroString"] = StringTag("")
        compound["zeroList"] = ListTag([])
        compound["zeroCompound"] = CompoundTag({})
        compound["zeroIntArray"] = IntArrayTag([])
        compound["zeroLongArray"] = LongArrayTag([])

        # empty inputs but numpy arrays for the array types
        compound["zeroNumpyByteArray"] = ByteArrayTag(numpy.array([]))
        compound["zeroNumpyIntArray"] = IntArrayTag(numpy.array([]))
        compound["zeroNumpyLongArray"] = LongArrayTag(numpy.array([]))

        # test the array types with some python data
        compound["listByteArray"] = ByteArrayTag([i for i in range(-128, 127)])
        compound["listIntArray"] = IntArrayTag([i for i in range(-400, 400)])
        compound["listLongArray"] = LongArrayTag([i for i in range(-400, 400)])

        # test the array types with numpy data of varying dtypes
        compound["numpyDtypeTestByteArray"] = ByteArrayTag(
            numpy.array([i for i in range(-128, 127)], dtype=int)
        )
        compound["numpyDtypeuTestByteArray"] = ByteArrayTag(
            numpy.array([i for i in range(-128, 127)], dtype=numpy.uint)
        )
        compound["numpyDtypeTestIntArray"] = IntArrayTag(
            numpy.array([i for i in range(-400, 400)], dtype=int)
        )
        compound["numpyDtypeuTestIntArray"] = IntArrayTag(
            numpy.array([i for i in range(-400, 400)], dtype=numpy.uint)
        )
        compound["numpyDtypeTestLongArray"] = LongArrayTag(
            numpy.array([i for i in range(-400, 400)], dtype=int)
        )
        compound["numpyDtypeuTestLongArray"] = LongArrayTag(
            numpy.array([i for i in range(-400, 400)], dtype=numpy.uint)
        )

        compound["numpyDtypedTestByteArray"] = ByteArrayTag(
            numpy.array([i for i in range(-128, 127)])
        )
        compound["numpyDtypedTestIntArray"] = IntArrayTag(
            numpy.array([i for i in range(-400, 400)])
        )
        compound["numpyDtypedTestLongArray"] = LongArrayTag(
            numpy.array([i for i in range(-400, 400)])
        )

        # test the extremes of the array types
        # byte array tested above
        compound["numpyExtremeTestIntArray"] = IntArrayTag(
            numpy.array([-(2**31), (2**31) - 1], dtype=int)
        )
        compound["numpyExtremeTestLongArray"] = LongArrayTag(
            numpy.array([-(2**63), (2**63) - 1], dtype="q")
        )

        compound["minByte"] = ByteTag(-128)
        compound["minShort"] = ShortTag(-(2**15))
        compound["minInt"] = IntTag(-(2**31))
        compound["minLong"] = LongTag(-(2**63))

        compound["maxByte"] = ByteTag(127)
        compound["maxShort"] = ShortTag(2**15 - 1)
        compound["maxInt"] = IntTag(2**31 - 1)
        compound["maxLong"] = LongTag(2**63 - 1)

        # these should either overflow when setting or error when saving. Test each and if it errors just comment it out
        compound["overflowByte"] = ByteTag(300)
        compound["underflowByte"] = ByteTag(-300)
        compound["overflowShort"] = ShortTag(2**16)
        compound["underflowShort"] = ShortTag(-(2**16))
        compound["overflowInt"] = IntTag(2**32)
        compound["underflowInt"] = IntTag(-(2**32))
        compound["overflowLong"] = LongTag(2**64)
        compound["underflowLong"] = LongTag(-(2**64))

        # compound['overflowByteArray'] = ByteArrayTag([-129, 128])
        # compound['overflowIntArray'] = IntArrayTag([-2**31-1, 2**31])
        # compound['overflowLongArray'] = LongArrayTag([-2**63-1, 2**63])

        # compound['overflowNumpyByteArray'] = ByteArrayTag(numpy.array([-129, 128]))
        # compound['overflowNumpyIntArray'] = IntArrayTag(numpy.array([-2**31-1, 2**31]))
        # compound['overflowNumpyLongArray'] = LongArrayTag(numpy.array([-2**63-1, 2**63]))

        # save then load back up and check the data matches

        os.makedirs("temp", exist_ok=True)
        named_compound.save_to(
            os.path.join("temp", "massive_nbt_test_big_endian.nbt"),
            compressed=False,
        )
        named_compound.save_to(
            os.path.join("temp", "massive_nbt_test_big_endian_compressed.nbt"),
            compressed=True,
        )
        named_compound.save_to(
            os.path.join("temp", "massive_nbt_test_little_endian.nbt"),
            compressed=False,
            little_endian=True,
        )

        test_be = load(os.path.join("temp", "massive_nbt_test_big_endian.nbt"))
        test_be_compressed = load(
            os.path.join("temp", "massive_nbt_test_big_endian_compressed.nbt")
        )
        test_le = load(
            os.path.join("temp", "massive_nbt_test_little_endian.nbt"),
            little_endian=True,
        )

        self.assertEqual(test_be, named_compound)
        self.assertEqual(test_be_compressed, named_compound)
        self.assertEqual(test_le, named_compound)


if __name__ == "__main__":
    unittest.main()
